/* Test portal engine */



#include <stdio.h>
#include <time.h>       // to compute frame rate

#include "../3dfc.h"



/* User function called during engine execution:
   move start room penguim
**********************************************************/
PTRVAL DoMove_Start ( CObject* poMyObject, PTRVAL pParam1, PTRVAL pParam2 )
{
   // move object
   
   static DWORD
      nState = 0;
   
   // get position/orientation for object
   CCameraSystem
      *poObjCamSys = &( (CComponentCameraSystem*) poMyObject->mRetComponent ( SLOT_REFSYS ) )->m_oCamSys;

   // update
   if ( nState < 100 ) {
      poObjCamSys->m_stPos.fY += 1;
      poObjCamSys->m_stPos.fX += 1;
   }   
   else
      if ( nState < 200 ) {   
         poObjCamSys->m_stPos.fY += 0.5;
         poObjCamSys->m_stPos.fX -= 1;
      }   
      else
         if ( nState < 300 ) {   
            poObjCamSys->m_stPos.fY -= 1.5;
         }   
         else
            nState = 0;

   nState ++;         
            
   return 0;
} 



/* User function called during engine execution:
   move big oct-room penguim
**********************************************************/
PTRVAL DoMove_Oct ( CObject* poMyObject, PTRVAL pParam1, PTRVAL pParam2 )
{
   static DWORD
      nState = 0;
      
   // setup initial direction
   if ( nState == 0 ) {

      tagfXYZ
         stDirection;
      stDirection.fX = 0;
      stDirection.fY = -1;
      stDirection.fZ = 5;
      poMyObject->mActivateComponent ( SLOT_MOVE, (PTRVAL) 3, (PTRVAL) &stDirection, (PTRVAL) 0 );

      nState ++;         
   }
         
   return 0;
} 



/* Only printf my address
**********************************************************/
void PrintContact ()
{
   printf ( "\n* If you have any questions or suggestions, make contact\n" );
   printf ( "     lebe@geocities.com\n" );
   printf ( "* Visit my web site to get the last version\n" );
   printf ( "     www.geocities.com/SiliconValley/Bay/1704\n" );
}



/* Make the socket connection, if working under Linux
**********************************************************/
void MakeSocketConnection ( CScenery& oScen, CEngine& oEngine, INT nNumArgs, char* szArgs [] )
{
   CString
      oszFlag;
   oszFlag = szArgs [ 2 ];
   
   // verify if this process is the client or the server
   if ( oszFlag.mCmpEq ( "-s" ) ) {

      // server configuration
      oEngine.m_stSocketInfo.chSocketUse = 's';
      oEngine.m_stSocketInfo.oszIP = "0.0.0.0";
      oEngine.m_stSocketInfo.dwPort = (DWORD) atoi ( szArgs [ 4 ] ); 
      oScen.mSetCurrentCamera ( (DWORD) atoi ( szArgs [ 3 ] ) );

      // REVIEW (bug do svgalib: soh aceita conexao em modo texto)
      printf ( "Waiting connection for 2 minutes...\n" );
      if ( ! oEngine.m_stSocketInfo.oSrvSoc.mCreate ( oEngine.m_stSocketInfo.oszIP, oEngine.m_stSocketInfo.dwPort, 5 ) )
         ABORT ( MSG_ERROR_SOCKET_CREATION, __FILE__, __LINE__ )
      
      if ( oEngine.m_stSocketInfo.oSrvSoc.mAccept ( 120, 0 ) )
         oEngine.m_stSocketInfo.bConnected = TRUE;
   }
   else {

      // client configuration
      oEngine.m_stSocketInfo.chSocketUse = 'c';
      oEngine.m_stSocketInfo.oszIP = szArgs [ 4 ];
      oEngine.m_stSocketInfo.dwPort = (DWORD) atoi ( szArgs [ 5 ] ); 
      oScen.mSetCurrentCamera ( (DWORD) atoi ( szArgs [ 3 ] ) );
      printf ( "Connecting...\n" );
      if ( ! oEngine.m_stSocketInfo.oCliSoc.mCreate ( oEngine.m_stSocketInfo.oszIP, oEngine.m_stSocketInfo.dwPort ) )
         ABORT ( MSG_ERROR_SOCKET_CREATION, __FILE__, __LINE__ )
      oEngine.m_stSocketInfo.bConnected = TRUE;      
   }
}



// compiling for X Window
#ifdef TRG_LINUXXWIN

/* Init and run engine under the X Window System.
Note: The code after oAPP.mExec () is not executed because
the exit is inside the CXWindowApp implementation.
**********************************************************/
void ExecSys ( CScenery& oScen, CEngine& oEngine, INT nNumArgs, char *szArgs [] )
{
   PrintContact ();

   // verify if use the socket system, if yes make connection
   if ( nNumArgs > 2 )
      MakeSocketConnection ( oScen, oEngine, nNumArgs, szArgs );

   // init window app
   oAPP.mCreate ( "Fly", 320, 200, nNumArgs, szArgs );
   // oAPP is a global instance for the X app interface, see port/linxwin/xwinapp.cc

   // set palette
   oAPP.mWritePal ( oScen.m_oPal );

   // execute
   oAPP.mExecApp ( oEngine );
}

#endif



// not compiling for X Window
#ifndef TRG_LINUXXWIN

/* Run engine without a window system like X Window
**********************************************************/
void ExecSys ( CScenery& oScen, CEngine& oEngine, 
               INT nNumArgs, char *szArgs [] )
{
   // define frame buffer
   CMem
      oFrameBuffer ( 320 * 200 );
   oEngine.mSetFrameBuffer ( 320, 200, oFrameBuffer );

   // verify if use the socket system, if yes make connection
   if ( nNumArgs > 2 )
      MakeSocketConnection ( oScen, oEngine, nNumArgs, szArgs );
   
   // make printf output visible in graphics mode (good for debug)
   /**( oScen.m_oPal.m_payMem + 7 * 3 + 0 ) = 50;
   * ( oScen.m_oPal.m_payMem + 7 * 3 + 1 ) = 50;
   * ( oScen.m_oPal.m_payMem + 7 * 3 + 2 ) = 50;*/

   // init output graphics system
   CScreen
      oS;
   oS.mCreate ( MODE_MCGA );
  
   // change palette
   oS.mSetPalette ( oScen.m_oPal );

   // init frame rate counter
   clock_t
      stClockStart = clock ();
   DWORD
      dwFrames = 0;

   // engine loop
   while ( oEngine.mRun () ) {

      // show frame buffer on screen
      oS.mShow ( oFrameBuffer );

      // count one frame
      dwFrames ++;
   }

   // return to text mode
   oS.mDestroy ();

   // end clock count and show message
   clock_t
      stClockEnd = clock ();
   printf ( MSG_FRAMES, (float) ( stClockEnd - stClockStart ) / CLOCKS_PER_SEC, dwFrames, (float) dwFrames / ( (float) ( stClockEnd - stClockStart ) / CLOCKS_PER_SEC ) );
   PrintContact ();
}   

#endif



/* The C entry point: parse command line and run engine
**********************************************************/
main ( INT nNumArgs, char *szArgs [] )
{
   printf ( "\n" );
   printf ( "3D Foundation Class version %s, Copyright (C) 1999 Leandro Bernsmuller\n", _3DFC_VERSION );
   printf ( "3D Foundation Class comes with ABSOLUTELY NO WARRANTY.\n" );
   printf ( "This is free software, and you are welcome to redistribute it\n" );
   printf ( "under certain conditions; read doc/license.txt for details.\n" );

   // verify command line options
   if ( nNumArgs == 1 ) {

      printf ( "\n3DFC demo - play a TXE scenery\n" );
      printf ( "\nUse: fly  scenery_file\n\n" );
      printf ( "Example: fly demo.txe\n\n" );
      printf ( "To exit    ESC.\n" );

      #if defined ( TRG_DOSDJGPP ) || defined ( TRG_LINUXXWIN )
      printf ( "To move    A, Z and cursor arrow keys pressed WITH ALT key pressed.\n" );
      printf ( "To rotate  7, 9 and cursor arrow keys pressed WITHOUT ALT key pressed.\n" );
      #endif

      #ifdef TRG_LINUXSVGA
      printf ( "To move    A, Z and cursor arrow keys.\n" );
      printf ( "To rotate  number keys: 7-9, 8-2, 4-6 with NUMLOCK ON.\n" );
      #endif

      #if defined ( TRG_LINUXSVGA ) || defined ( TRG_LINUXXWIN )
      printf ( "\nLinux special options:\n" );
      printf ( "Use: fly  scenery_file  [ <-s CameraID Port> | <-c CameraID IPaddress Port> ]\n\n" );
      printf ( " -s         make this process work as server, wait connection\n" );
      printf ( " -c         make this process work as client, make connection\n" );
      printf ( " CameraID   the camera object, what scenery object are me\n" );
      printf ( " IPaddress  destin IP address\n" );
      printf ( " Port       the port, client and server must use the same value.\n" );
      printf ( "Example: fly demo.txe -c 1 127.0.0.1 8189    load demo.txe, client mode\n" );
      #endif

      PrintContact ();
      return 0;
   }
      
   // define scenery file to load
   CString
      oszFile ( szArgs [ 1 ] );

   printf ( "\n\nLoading scenery... be pacient at first time...\n\n" );

   // load scenery
   CScenery
      oScen;
   // try load and make verify load error
   if ( ! oScen.mCreate ( oszFile ) ) {
      printf ( "Scenery load error.\n" );
      return 0;
   }
   
   // add user functions
   printf ( "\n" );
   printf ("Adding user function %d... result %d\n", 0, oScen.mAddFunction ( (PTRVAL) DoMove_Start, 0 ) );
   printf ("Adding user function %d... result %d\n", 1, oScen.mAddFunction ( (PTRVAL) DoMove_Oct, 1 ) );

   // init engine
   CEngine
      oEngine ( oScen );

   ExecSys ( oScen, oEngine, nNumArgs, szArgs );
}
